function [data, E, Diff, dt_hist, output]= Harmonic_ls_GBB_mix(data, pars)
%-------------------------------------------------------------------------
% Barzilar-Borwein's method beginning with a few monotone line searchs 
%
% Reference: 
%   Goldfarb, Donald and Wen, Zaiwen and Yin, Wotao
%   A Curvilinear Search Method for the p-Harmonic Flow on Sphere
%
% Author: Zaiwen Wen, Wotao Yin
%   Version 1.0 .... 2007/11
%-------------------------------------------------------------------------



% % Call monotone line search for a few steps
%
% [data, E, Diff, dt_hist, output]= Harmonic_ls(data, pars);
%
%
% % then Call ls-BB
% [data, E, Diff, dt_hist, output]= Harmonic_bb(data, pars);


%% Size information
% [m,n]=size(u);
% if any([m,n]~=size(v))  error('u and v have different sizes.'); end;

data = CheckUserData( data );


if ~isfield(pars, 'CheckOptimal')
    pars.CheckOptimal = 1;
end

if ~isfield(pars, 'GradEPS')
    pars.GradEPS = 1e-3;
end

if ~isfield(pars, 'maxiter')
    pars.maxiter = 5000;
end

nls1 = 20;
if ~isfield(pars, 'n_ls_monotone')
    pars.n_ls_monotone = nls1;
end

% run monotone line search at least nls1 iterations
if pars.n_ls_monotone < nls1;
    pars.n_ls_monotone = nls1;
end


% choice of nonmontone line search scheme
if isfield(pars, 'method')
   if strcmp(pars.method, 'Raydan') ~=1 && strcmp(pars.method, 'HongChao') ~=1 
       pars.method = 'Raydan';
   end
else
     pars.method = 'HongChao';
     pars.method = 'Raydan';
end

% termination rule
if isfield(pars, 'xtol')
   if pars.xtol < 0 || pars.xtol > 1
        pars.xtol = 1e-4;
   end
else
    pars.xtol = 1e-4;
end
    
if isfield(pars, 'gtol')
   if pars.gtol < 0 || pars.gtol > 1
        pars.gtol = 1e-4;
   end
else
    pars.gtol = 1e-4;
end

% parameters for control the linear approximation in line search
if isfield(pars, 'rho')
   if pars.rho < 0 || pars.rho > 1
        pars.rho = 1e-4;
   end
else
    pars.rho = 1e-4;
end

% factor for decreasing the step size in the backtracking line search
if isfield(pars, 'eta')
   if pars.eta < 0 || pars.eta > 1
        pars.eta = 0.1;
   end
else
    pars.eta = 0.1;
end

% parameters for updating C by HongChao, Zhang
if isfield(pars, 'gamma')
   if pars.gamma < 0 || pars.gamma > 1
        pars.gamma = 0.85;
   end
else
    pars.gamma = 0.85;
end

% parameters for the  nonmontone line search by Raydan
if ~isfield(pars, 'M')
    pars.M = 10;
end

if ~isfield(pars, 'STPEPS')
    pars.STPEPS = 1e-10;
end


% iteration history
E = zeros(1,(pars.maxiter+1));
Diff = zeros(1,(pars.maxiter+1));
dt_hist = zeros(1,pars.maxiter + 1);


% copy initial solution to the working space
data.u = data.u0;
data.v = data.v0;
if data.dimS == 2
    data.w = data.w0;
end

% compute finite difference information
data = feval(pars.DiffCommon, data);

% compute objective function value
E(1) = feval(pars.EnergyFun, data);
% Diff(1) = feval(pars.DistanceUV, data);
dt_hist(1) = 0;

% compute the gradient at current point
data = feval(pars.EnergyGrad, data);

output.Total_NFE = 1;
if pars.CheckOptimal == 1
    output.optimal = 0;
end

% parameters for line search
parsls.maxiter = 10;

parsls.ftol = 1e-4;
parsls.gtol = 0.9;
dt = 1e-2;


%% Print iteration header if debug == 1
if (pars.debug == 1)
    fid = 1;
    fprintf(fid, '----------- Projected Gradient Method with Line search----------- \n');
    fprintf(fid, '%4s \t %10s \t %10s \t %5s \t %9s \t %7s \n', 'Iter', 'dt', 'E(i)', 'Exit', 'funcCount', 'ls-Iter');
    fprintf(fid, '%4d \t %e \t %e \t %5d \t %5d	\t %6d	\n', 1, 0, E(1), 0, 0, 0);
    %OUTPUT
end


%% Evolution
for ind = 2 : (pars.n_ls_monotone)
% for ind = 2 : (pars.maxiter+1)
    % begin line search
    %     clear workls;

    % must set "work.task = 1"
    workls.task =1;

    % copy U , GradU, Div to temporary storage, this is because of the
    % data structure we use.  We need the directional derivaitve information at
    % the new trial point.
    %    d (E(U(t))) = grad( E(U(t)))' U'(t)
    % We copy "U , GradU, Div" to "U1 , GradU, Div1"
    % We copy "data" to "datals"
    % Then work on datals, but datals.U1 , GradU1, Div1 are information
    % relate to U, not to U(t)
    %
    % Finally, we copy datals to data to get U_{k+1}
    %

    data = feval(pars.CopyUInfoToTmp, data);

    % objective function value, copy from previous computation
    energyt = E(ind-1);
    energy_deriv = feval(pars.Energy_Deriv, 0, data);
    output.GradNorm = sqrt(abs( energy_deriv ));


    if pars.CheckOptimal == 1
        % if satisfy termination rule
        %if output.GradNorm <= max(100*pars.GradEPS *( 1 + E(ind-1) ), 100*pars.GradEPS);
         % if output.GradNorm <= max(100*pars.GradEPS *( 1 + E(ind-1) ), 100*pars.GradEPS);
        if output.GradNorm <= pars.GradEPS
            output.TotalIter = ind-1;
            output.Fval = energyt;
            output.optimal = 1;
            %                 output.Energy_Deriv = energy_deriv;
            return;
           
        end
    end


    output_ls.iter = 0;    output_ls.nfe = 0;

    %       call line search, reverse communication
    while 1

        [dt, energyt, energy_deriv, parsls, workls] = ls_csrch(dt, energyt, energy_deriv , parsls , workls);

        % Evaluate the function and the gradient at stp
        if (workls.task == 2)

            %% now, generate the new trial point
            % compute the trial point Ut
            datals = feval(pars.UpdateUScheme, data, dt);

            % copy the trial point Ut to U
            datals = feval(pars.CopyUtToU, datals);

            %% compute function value and gradient
            % compute error respect to the optimal solution
            datals = feval(pars.DiffCommon, datals);

            energyt = feval(pars.EnergyFun, datals);

            % compute gradient at current point, must have called DiffCommon before
            datals = feval(pars.EnergyGrad, datals);

            energy_deriv = feval(pars.Energy_Deriv, dt, datals);

            output_ls.iter = output_ls.iter + 1;      output_ls.nfe = output_ls.nfe + 1;

        else  % exit
            break
        end
    end

%     %--------------------------------------------------------------------
%     % Data preparation for ls-BB
%     if ind == pars.n_ls_monotone - 1
%         % copy U and grad(Ep(U)) to temporary storage
%         % data = feval(pars.CopyUInfoToTmp, data);
%         dataold = data;
%     end
%     %--------------------------------------------------------------------

    data = datals;
    output.Total_NFE = output.Total_NFE + output_ls.nfe;
    output_ls.exitflag = workls.task;

    E(ind) = energyt;
  
    % compute error respect to the optimal solution

%     Diff(ind) = feval(pars.DistanceUV, data);
    dt_hist(ind) = dt;

   
    if (pars.debug == 1)
        fprintf(fid, '%4d \t %e \t %e \t %5d \t %5d	\t %6d	\t %s \n', ind,...
            dt, E(ind), output_ls.exitflag, output_ls.nfe, output_ls.iter, workls.msg);
        %OUTPUT
    end
end

nls = ind;
fprintf('\nnumber of iterations in the first stage: %d\n', nls);
fprintf('E: %4.3e, ||G||: %4.3e, \n', E(ind), output.GradNorm);

%%-------------------------------------------------------------------------
% second stage, BB

% copy parameters
xtol = pars.xtol;
gtol = pars.gtol;
rho  = pars.rho;
M     = pars.M;
STPEPS = pars.STPEPS;
eta   = pars.eta;
gamma = pars.gamma;


% set the active grid points ( Dirichlet problem )
data.idx = 2:data.m-1;      idx = data.idx;
data.idy = 2:data.n-1;      idy = data.idy;

alpha = 1e3;
Q = 1; 
C = E(ind);
% C = max(E);

%% main iteration
% for ind = pars.n_ls_monotone+1 : pars.maxiter+1
for ind = nls+1 : pars.maxiter+1
    
    % compute a stepsize using bb method
    s_u_k_1 = data.u(idx,idy) - data.u1(idx,idy);
    s_v_k_1 = data.v(idx,idy) - data.v1(idx,idy);

    y_u_k_1 = data.Gradu - data.Gradu1;
    y_v_k_1 = data.Gradv - data.Gradv1;


    sy = sum(sum(s_u_k_1.*y_u_k_1)) + sum(sum(s_v_k_1.*y_v_k_1));

    if data.dimS == 2
        s_w_k_1 = data.w(idx,idy) - data.w1(idx,idy);
        y_w_k_1 = data.Gradw - data.Gradw1;
        sy = sy + sum(sum(s_w_k_1.*y_w_k_1));
    end

    if (pars.bb == 1) || ( pars.bb == 3 && mod(ind,2) == 0 )

        yy = sum(sum(y_u_k_1.*y_u_k_1 + y_v_k_1.*y_v_k_1));

        if data.dimS == 2
            yy = yy +  sum(sum(y_w_k_1.*y_w_k_1));
        end
        dt = sy / yy;
    end

    if pars.bb == 2 || ( pars.bb == 3 && mod(ind,2) ~= 0 )
        ss = sum(sum(s_u_k_1.*s_u_k_1 + s_v_k_1.*s_v_k_1));

        if data.dimS == 2
            ss = ss +  sum(sum(s_w_k_1.*s_w_k_1));
        end
        dt = ss / sy;

    end

%    % scale step size
%     if dt <= STPEPS || dt >= 1/STPEPS
%         if  output.GradNorm > 1
%             delta = 1;
%         elseif output.GradNorm >= 1e-5
%             delta = 1/output.GradNorm;
%         else
%             delta = 1e5;
%         end
%         dt = delta;
%     end
        
    dt0 = dt;
    
    output_ls.iter = 0;    output_ls.nfe = 0;
    lsconst = rho*(output.GradNorm)^2;
    % call nonmonotone line search
    while 1
        %% now, generate the new trial point
        % compute the trial point Ut
        datals = feval(pars.UpdateUScheme, data, dt);

        % copy the trial point Ut to U
        datals = feval(pars.CopyUtToU, datals);

        %% compute function value and gradient
        % compute error respect to the optimal solution
        datals = feval(pars.DiffCommon, datals);

        energyt = feval(pars.EnergyFun, datals);


        fprintf('nls: %d, dt0:%4.3e, dt:%4.3e, E: %4.3e \n', output_ls.iter, dt0, dt, energyt)
        
        stp = dt;        
        if energyt <= C - dt*lsconst || output_ls.iter >= 5
            break
        end
        dt = eta*dt;
        output_ls.iter = output_ls.iter + 1;      output_ls.nfe = output_ls.nfe + 1;
        
    end
    
    
    dt_hist(ind) = dt;
    
    
%     fprintf('nls: %d, dt0:%4.3e, dt:%4.3e, E: %4.3e \n', output_ls.iter, dt0, dt, energyt)
    
    
    % compute gradient at current point, must have called DiffCommon before
    datals = feval(pars.EnergyGrad, datals);
    energy_deriv = feval(pars.Energy_Deriv, 0, datals);
    output.GradNorm = sqrt(abs( energy_deriv ));            
    
    data = datals;
    output.Total_NFE = output.Total_NFE + output_ls.nfe;
    output_ls.exitflag = workls.task;

    E(ind) = energyt;    
    dt_hist(ind) = dt;
    
    if pars.CheckOptimal == 1
        % if satisfy termination rule
%         if output.GradNorm <= pars.GradEPS *( 1 + E(ind-1) )
        if output.GradNorm <= pars.GradEPS
            output.TotalIter = ind;
            output.Fval = E(ind);
            output.optimal = 1;
            return;
        end
    end    

    % update for next iteration

    
    % By Raydan
    if strcmp(pars.method, 'Raydan')
        C = max( E( ind - min(ind-1, M): ind) );
    elseif strcmp(pars.method, 'Hongchao')
        % by HongChao Zhang
        Qp = Q; Q = gamma*Qp + 1; C = (gamma*Qp*C + E(ind))/Q;
    end
    


    % output information
    if (pars.debug == 1)
        fprintf(fid, '%4d \t %e \t %e \t %e\n', ind, dt, E(ind), output.GradNorm);
    end

end

output.optimal = 0;
output.Fval = E(pars.maxiter+1);
output.TotalIter = pars.maxiter;
% output.Energy_Deriv = energy_deriv;

% fprintf('Deriv: %f \n', output.Energy_Deriv);

